==> commerce_shipping_best_rate-1.0.0.tar.gz: commerce_shipping_best_rate/src/EventSubscriber/ShippingRatesSubscriber.php <==
<?php

namespace Drupal\commerce_shipping_best_rate\EventSubscriber;

use Drupal\commerce_price\Price;
use Drupal\commerce_shipping\Event\ShippingEvents;
use Drupal\commerce_shipping\Event\ShippingRatesEvent;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Shipping rates event subscriber for best rate plugin.
 *
 * Removes rates (services) that were configured in best_rate shipping methods
 * and updates the prices of the best_rate shipping methods using the lowest
 * cost service.
 */
class ShippingRatesSubscriber implements EventSubscriberInterface {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new ShippingRatesSubscriber object.
   *
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(AccountInterface $current_user, EntityTypeManagerInterface $entity_type_manager) {
    $this->currentUser = $current_user;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      ShippingEvents::SHIPPING_RATES => 'filterBestRate',
    ];
  }

  /**
   * Filter and replace configured rates.
   *
   * @param \Drupal\commerce_shipping\Event\ShippingRatesEvent $event
   *   The event.
   */
  public function filterBestRate(ShippingRatesEvent $event) {
    // Store the removed rates in a static variable which persists for the
    // entire recalculation request.
    $saved_rates = &drupal_static(__FUNCTION__ . '_saved_rates', []);

    // Store the configured best rate method configuration in static variables.
    $best_rate_groups = &drupal_static(__FUNCTION__ . '_best_rate_groups', []);
    $best_rate_excluded = &drupal_static(__FUNCTION__ . '_best_rate_excluded', []);
    $best_rate_display = &drupal_static(__FUNCTION__ . '_best_rate_display', []);

    // Get the list of configured rate ids from the best rate shipping methods.
    if (empty($best_rate_groups)) {
      $best_rate_storage = $this->entityTypeManager->getStorage('commerce_shipping_method');
      $best_rate_query = $best_rate_storage->getQuery()
        ->accessCheck(TRUE)
        ->condition('status', 1)
        ->condition('plugin', 'best_rate')
        ->sort('shipping_method_id', 'ASC')
        ->execute();
      $best_rate_methods = $best_rate_storage->loadMultiple($best_rate_query);

      // Get the current user's roles.
      $user_roles = $this->currentUser->getRoles();

      // Create arrays containing best rate configuration options keyed by the
      // best rate method id they belong to.
      foreach ($best_rate_methods as $best_rate_method) {
        // Create an array of configured rate ids.
        $best_rate_groups[$best_rate_method->id()] = $best_rate_method->getPlugin()->getServicesGroup();
        // Compare the current user's roles with the configured excluded roles
        // and set to true if there is a match.
        $best_rate_excluded[$best_rate_method->id()] = (count(array_intersect($best_rate_method->getPlugin()->getExcludedRoles(), $user_roles)) > 0 ? TRUE : FALSE);
        // Set display of best rate to TRUE if it either not excluded or
        // showing both is enabled.
        if ($best_rate_excluded[$best_rate_method->id()]) {
          $best_rate_display[$best_rate_method->id()] = ($best_rate_method->getPlugin()->getExcludedRolesDisplay() ? TRUE : FALSE);
        }
        else {
          $best_rate_display[$best_rate_method->id()] = TRUE;
        }
      }
    }

    $rates = $event->getRates();
    $rates_changed = FALSE;

    $shipping_method = $event->getShippingMethod();
    $shipping_method_id = $shipping_method->id();
    $shipping_method_plugin_id = $shipping_method->getPlugin()->getPluginId();

    if ($shipping_method_plugin_id == 'best_rate') {
      if (array_key_exists($shipping_method_id, $saved_rates) && $best_rate_display[$shipping_method_id]) {
        /** @var \Drupal\commerce_price\Price $best_price */
        $best_price = new Price('0.00', 'USD');

        // Search for the lowest price.
        foreach ($saved_rates[$shipping_method_id] as $key => $saved_rate) {
          $price = $saved_rate->getAmount();

          // If this is the first iteration then skip comparison.
          if ($key === array_key_first($saved_rates[$shipping_method_id])) {
            $best_price = $price;
          }
          // Change the price if it is lower than the current best price.
          elseif ($price->lessThan($best_price)) {
            $best_price = $price;
          }
        }

        $rates[0]->setOriginalAmount($best_price);
        $rates[0]->setAmount($best_price);
      }
      // Delete best_rate rate if there were no matching saved_rates.
      else {
        $rates = [];
        $rates_changed = TRUE;
      }
    }
    // Search for rates with ids that match those configured in
    // best rate methods.
    else {
      foreach ($rates as $key => $rate) {
        $rate_id = $rate->getId();
        foreach ($best_rate_groups as $method_key => $best_rate_group) {
          if (in_array($rate_id, $best_rate_group)) {
            $saved_rates[$method_key][] = $rate;
            // Remove the rate unless best rate is excluded for this user.
            if (!$best_rate_excluded[$method_key]) {
              unset($rates[$key]);
              $rates_changed = TRUE;
            }
          }
        }
      }
    }

    // Update the rates if they have been modified.
    if ($rates_changed) {
      $event->setRates($rates);
    }
  }

}

==> commerce_shipping_best_rate-1.1.0.tar.gz: commerce_shipping_best_rate/src/EventSubscriber/ShippingRatesSubscriber.php <==
<?php

declare(strict_types=1);

namespace Drupal\commerce_shipping_best_rate\EventSubscriber;

use Drupal\commerce_price\Price;
use Drupal\commerce_shipping\Event\ShippingEvents;
use Drupal\commerce_shipping\Event\ShippingRatesEvent;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Shipping rates event subscriber for best rate plugin.
 *
 * Removes rates (services) that were configured in best_rate shipping methods
 * and updates the prices of the best_rate shipping methods using the lowest
 * cost service.
 */
class ShippingRatesSubscriber implements EventSubscriberInterface {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new ShippingRatesSubscriber object.
   *
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(AccountInterface $current_user, EntityTypeManagerInterface $entity_type_manager) {
    $this->currentUser = $current_user;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      ShippingEvents::SHIPPING_RATES => 'filterBestRate',
    ];
  }

  /**
   * Filter and replace configured rates.
   *
   * @param \Drupal\commerce_shipping\Event\ShippingRatesEvent $event
   *   The event.
   */
  public function filterBestRate(ShippingRatesEvent $event): void {
    // Store the removed rates in a static variable which persists for the
    // entire recalculation request.
    $saved_rates = &drupal_static(__FUNCTION__ . '_saved_rates', []);

    // Store the configured best rate method configuration in static variables.
    $best_rate_groups = &drupal_static(__FUNCTION__ . '_best_rate_groups', []);
    $best_rate_excluded = &drupal_static(__FUNCTION__ . '_best_rate_excluded', []);
    $best_rate_display = &drupal_static(__FUNCTION__ . '_best_rate_display', []);

    // Get the list of configured rate ids from the best rate shipping methods.
    if (empty($best_rate_groups)) {
      $best_rate_storage = $this->entityTypeManager->getStorage('commerce_shipping_method');
      $best_rate_query = $best_rate_storage->getQuery()
        ->accessCheck(TRUE)
        ->condition('status', 1)
        ->condition('plugin', 'best_rate')
        ->sort('shipping_method_id', 'ASC')
        ->execute();
      /** @var \Drupal\commerce_shipping\Entity\ShippingMethod[] $best_rate_methods */
      $best_rate_methods = $best_rate_storage->loadMultiple($best_rate_query);

      // Get the current user's roles.
      $user_roles = $this->currentUser->getRoles();

      // Create arrays containing best rate configuration options keyed by the
      // best rate method id they belong to.
      foreach ($best_rate_methods as $best_rate_method) {
        /** @var \Drupal\commerce_shipping_best_rate\Plugin\Commerce\ShippingMethod\BestRate $best_rate_plugin */
        $best_rate_plugin = $best_rate_method->getPlugin();
        // Create an array of configured rate ids.
        $best_rate_groups[$best_rate_method->id()] = $best_rate_plugin->getServicesGroup();
        // Compare the current user's roles with the configured excluded roles
        // and set to true if there is a match.
        $best_rate_excluded[$best_rate_method->id()] = (count(array_intersect($best_rate_plugin->getExcludedRoles(), $user_roles)) > 0 ? TRUE : FALSE);
        // Set display of best rate to TRUE if it either not excluded or
        // showing both is enabled.
        if ($best_rate_excluded[$best_rate_method->id()]) {
          $best_rate_display[$best_rate_method->id()] = ($best_rate_plugin->getExcludedRolesDisplay() ? TRUE : FALSE);
        }
        else {
          $best_rate_display[$best_rate_method->id()] = TRUE;
        }
      }
    }

    $rates = $event->getRates();
    $rates_changed = FALSE;

    /** @var \Drupal\commerce_shipping\Entity\ShippingMethod $shipping_method */
    $shipping_method = $event->getShippingMethod();
    $shipping_method_id = $shipping_method->id();
    $shipping_method_plugin = $shipping_method->getPlugin();
    $shipping_method_plugin_id = $shipping_method_plugin->getPluginId();

    if ($shipping_method_plugin_id == 'best_rate') {
      if (array_key_exists($shipping_method_id, $saved_rates) && $best_rate_display[$shipping_method_id]) {
        /** @var \Drupal\commerce_price\Price $best_price */
        $best_price = new Price('0.00', 'USD');
        $best_rate = NULL;

        // Search for the lowest price.
        foreach ($saved_rates[$shipping_method_id] as $key => $saved_rate) {
          $price = $saved_rate->getAmount();

          // If this is the first iteration then skip comparison.
          if ($key === array_key_first($saved_rates[$shipping_method_id])) {
            $best_rate = $saved_rate;
            $best_price = $price;
          }
          // Change the price / rate if it is lower than the current best price:
          elseif ($price->lessThan($best_price)) {
            $best_price = $price;
            $best_rate = $saved_rate;
          }
        }
        // Replace the best rate with the chosen rate if configured, else update
        // the price of the best rate.
        if ($shipping_method_plugin->getConfiguration()['reference_real_rate'] ?? FALSE) {
          $rates[0] = $best_rate;
          $rates_changed = TRUE;
        }
        else {
          $rates[0]->setOriginalAmount($best_price);
          $rates[0]->setAmount($best_price);
        }

      }
      // Delete best_rate rate if there were no matching saved_rates.
      else {
        $rates = [];
        $rates_changed = TRUE;
      }
    }
    // Search for rates with ids that match those configured in
    // best rate methods.
    else {
      foreach ($rates as $key => $rate) {
        $rate_id = $rate->getId();
        foreach ($best_rate_groups as $method_key => $best_rate_group) {
          if (in_array($rate_id, $best_rate_group)) {
            $saved_rates[$method_key][] = $rate;
            // Remove the rate unless best rate is excluded for this user.
            if (!$best_rate_excluded[$method_key]) {
              unset($rates[$key]);
              $rates_changed = TRUE;
            }
          }
        }
      }
    }

    // Update the rates if they have been modified.
    if ($rates_changed) {
      $event->setRates($rates);
    }
  }

}
